/*****************************************************************************
 *
 * Europa Firmware Update Utility - Windows version
 * Copyright 2000-2001, Synthcom Systems, Inc.
 *
 * Version 1.00
 * Compiled with Microsoft Visual C++ 6.0 service pack 2
 *
 *****************************************************************************/

#include <windows.h>
#include <stdio.h>
#include <assert.h>
#include <stdarg.h>
#include "hosttype.h"
#include "resource.h"
#include "../core/europa.h"
#include "../core/efile.h"
#include "../core/emidi.h"
#include "../core/etext.h"
#include <mmsystem.h>
#include <commdlg.h>
#include <commctrl.h>

#include <winreg.h>

static HWND sg_hDialog;
static HWND sg_hProgressDialog;
static OPENFILENAME sg_sOpenFilename;
static BOOL sg_bFileDialogUp = FALSE;
static UINT8 sg_u8EFIFilename[250];
static UINT32 sg_u32InputBus = 0xffffffff;
static UINT32 sg_u32OutputBus = 0xffffffff;
static BOOL sg_bFilenameSelected = FALSE;
static BOOL sg_bConnected = FALSE;
static HMIDIIN sg_hMIDIInHandle = 0;
static HMIDIOUT sg_hMIDIOutHandle = 0;
static MIDIHDR sg_hMIDIInHdr;
static UINT8 sg_u8SysexReceive[1000];
static HINSTANCE sg_hInstance;
static struct sEuropaStatus sStatus;

/*****************************************************************************
 *
 * Procedure  : HostPostError
 *
 * Input      : Pointer to error string to display.
 *
 * Output     : A dialog box with an "OK" pushbutton is shown and won't return
 *				until OK is pressed.
 *
 * Description:
 *
 * This routine will post a message box (with the error code provided) and an
 * "OK" button. It won't return until the OK has been pressed by the user.
 *
 *****************************************************************************/

void HostPostError(UINT8 *pu8ErrorString)
{
	MessageBox(sg_hDialog, pu8ErrorString, "Europa Firmware Update Utility Error", MB_OK);
}

/*****************************************************************************
 *
 * Procedure  : HostMessageBox
 *
 * Input      : Pointer to message query to display (yes/no)
 *
 * Output     : TRUE If YES was selected, otherwise FALSE
 *
 * Description:
 *
 * Displays a message box similar to HostPostError() but asks for a Yes/No
 * response and doesn't return until it gets one or the other.
 *
 *****************************************************************************/

BOOL HostMessageBox(UINT8 *pu8Message)
{	
	int result;

	result = MessageBox(sg_hDialog, (LPCTSTR) pu8Message, "Europa Firmware Update Utility", MB_YESNO | MB_DEFBUTTON2);

	if (IDYES == result)
	{
		return(TRUE);
	}
	else
	{
		return(FALSE);
	}
}

/*****************************************************************************
 *
 * Procedure  : HostProbeMIDIBuses()
 *
 * Input      : Nothing
 *
 * Output     : # Of MIDI OUT devices
 *
 * Description:
 *
 * Looks through all the possible MIDI in and out buses and adds them to the
 * MIDI bus dialog box combo box list.
 *
 *****************************************************************************/

static UINT32 HostProbeMIDIBuses(void)
{
	MMRESULT mmresult;
	MIDIINCAPS pmic;
	MIDIOUTCAPS pmoc;
	UINT32 u32Count;
	UINT32 u32Loop;
	INT32 s32DialogReturn;
	
	// Get # of MIDI IN devices

	u32Count = midiInGetNumDevs();

	// If there are no MIDI IN devices, bail!

	if (0 == u32Count)
	{
		HostPostError("No MIDI input buses available! Please install your MIDI interface's Windows driver");
		return(0);
	}

	s32DialogReturn = SendDlgItemMessage(sg_hDialog, IDC_MIDI_IN_BUS, CB_ADDSTRING, (UINT) -1, (long) "(Choose a MIDI port)");
	s32DialogReturn = SendDlgItemMessage(sg_hDialog, IDC_MIDI_IN_BUS, CB_SETCURSEL, 0, 0);

	// Loop through them all and get their info!

	for (u32Loop = 0; u32Loop < u32Count; u32Loop++)
	{
		mmresult = midiInGetDevCaps(u32Loop,
									&pmic,
									sizeof(pmic));

		if (MMSYSERR_NOERROR == mmresult)
		{
			s32DialogReturn = SendDlgItemMessage(sg_hDialog, IDC_MIDI_IN_BUS, CB_ADDSTRING, (UINT) -1, (long) pmic.szPname);
		}
	}

	// Get # of MIDI OUT devices

	u32Count = midiOutGetNumDevs();

	// If there are no MIDI OUT devices, bail!

	if (0 == u32Count)
	{
		HostPostError("No MIDI output buses available! Please install your MIDI card's Windows driver");
		return(0);
	}

	s32DialogReturn = SendDlgItemMessage(sg_hDialog, IDC_MIDI_OUT_BUS, CB_ADDSTRING, (UINT) -1, (long) "(Choose a MIDI port)");
	s32DialogReturn = SendDlgItemMessage(sg_hDialog, IDC_MIDI_OUT_BUS, CB_SETCURSEL, 0, 0);

	// Loop through them all and get their info!

	for (u32Loop = 0; u32Loop < u32Count; u32Loop++)
	{
		mmresult = midiOutGetDevCaps(u32Loop,
									 &pmoc,
									 sizeof(pmoc));

		if (MMSYSERR_NOERROR == mmresult)
		{
			s32DialogReturn = SendDlgItemMessage(sg_hDialog, IDC_MIDI_OUT_BUS, CB_ADDSTRING, (UINT) -1, (long) pmoc.szPname);
		}
	}

	// Must be a nonzero value

	return(u32Count);
}

/*****************************************************************************
 *
 * Procedure  : SetUpdateButton
 *
 * Input      : TRUE If the UPDATE button should be active, otherwise FALSE
 *
 * Output     : None
 *
 * Description:
 *
 * Enables or disables the dialog box's UPDATE button
 *
 *****************************************************************************/

static void SetUpdateButton(BOOL bEnabled)
{
	HWND hWndCtrl;

	hWndCtrl = GetDlgItem(sg_hDialog, ID_UPDATE);
	EnableWindow(hWndCtrl, bEnabled);
}

/*****************************************************************************
 *
 * Procedure  : SetFindImageButton
 *
 * Input      : TRUE If the FIND IMAGE button should be active, otherwise FALSE
 *
 * Output     : None
 *
 * Description:
 *
 * Enables or disables the dialog box's FIND IMAGE button
 *
 *****************************************************************************/

static void SetFindImageButton(BOOL bEnabled)
{
	HWND hWndCtrl;

	hWndCtrl = GetDlgItem(sg_hDialog, ID_FIND_IMAGE);
	EnableWindow(hWndCtrl, bEnabled);
}

/*****************************************************************************
 *
 * Procedure  : SetConnectButton
 *
 * Input      : TRUE If the CONNECT button should be active, otherwise FALSE
 *
 * Output     : None
 *
 * Description:
 *
 * Enables or disables the dialog box's CONNECT button
 *
 *****************************************************************************/

static void SetConnectButton(BOOL bEnabled)
{
	HWND hWndCtrl;

	hWndCtrl = GetDlgItem(sg_hDialog, ID_CONNECT);
	EnableWindow(hWndCtrl, bEnabled);
}

/*****************************************************************************
 *
 * Procedure  : FatalError
 *
 * Input      : printf-like list of formal parameters
 *
 * Output     : Nothing
 *
 * Description:
 *
 * Formats and displays a printf-like set of formal parameters, waits for the
 * user to acknowledge it by clicking OK, and returns.
 *
 *****************************************************************************/

static void FatalError(UINT8 *pszFormat, ... )
{
	va_list ap;
	UINT8 szTemp[350];

	if (pszFormat)
	{
		va_start(ap, pszFormat);
		vsprintf(szTemp, pszFormat, ap);
		va_end(ap);

		HostMessageBox(szTemp);
	}
}

/*****************************************************************************
 *
 * Procedure  : OpenFileDialog
 *
 * Input      : Nothing
 *
 * Output     : Nothing
 *
 * Description:
 *
 * Displays a file dialog box and copies the selected filename (if any) into
 * sg_u8EFIFilename.
 *
 *****************************************************************************/

static void OpenFileDialog(void)
{
	INT32 s32DialogReturn;
	BOOL bReturn;

	if (sg_bFileDialogUp)
	{
		return;
	}

	ZeroMemory(&sg_sOpenFilename, sizeof(sg_sOpenFilename));
	ZeroMemory(&sg_u8EFIFilename, sizeof(sg_u8EFIFilename));

	sg_sOpenFilename.lStructSize = sizeof(sg_sOpenFilename);
	sg_sOpenFilename.hwndOwner = sg_hDialog;
	sg_sOpenFilename.hInstance = 0;
	sg_sOpenFilename.lpstrFilter = "Europa Firmware Image (*.efi)\0*.efi\0";
	sg_sOpenFilename.lpstrTitle = "Europa Firmware Update Utility file selector";
	sg_sOpenFilename.lpstrFile = sg_u8EFIFilename;
	sg_sOpenFilename.nMaxFile = sizeof(sg_u8EFIFilename) - 1;
	sg_sOpenFilename.Flags = OFN_HIDEREADONLY;

	sg_bFileDialogUp = TRUE;
	bReturn = GetOpenFileName(&sg_sOpenFilename);
	sg_bFileDialogUp = FALSE;

	if (bReturn)
	{
		s32DialogReturn = SendDlgItemMessage(sg_hDialog, 
											 IDC_FILENAME, 
											 WM_SETTEXT, (UINT) -1, (long) sg_u8EFIFilename);
	}
}


/*****************************************************************************
 *
 * Procedure  : MIDIReceive
 *
 * Input      : MIDI Receive port, MIDI message, instance, and parameters
 *
 * Output     : Nothing
 *
 * Description:
 *
 * Called when Windows has received a MIDI message (MIDI receive callback)
 *
 *****************************************************************************/

void CALLBACK MIDIReceive(HMIDIIN hMidiIn,  
						  UINT wMsg,        
						  DWORD dwInstance, 
						  DWORD dwParam1,   
						  DWORD dwParam2)
{
	UINT32 u32Loop;
	MMRESULT MIDIResult;

	// We only care about system exclusive messages

	if (MIM_LONGDATA == wMsg)
	{
		for (u32Loop = 0; u32Loop < sg_hMIDIInHdr.dwBytesRecorded; u32Loop++)
		{
			EuropaReceiveMIDIByte(sg_u8SysexReceive[u32Loop]);
		}

		// Only reallocate another MIDI IN buffer if the dwBytesRecorded is nonzero. If it's
		// zero, this is Windows' way of telling us that it's shutting down. Thank you
		// Microsoft for documenting this - NOT!

		if (sg_hMIDIInHdr.dwBytesRecorded)
		{
			memset(sg_u8SysexReceive, 0, sizeof(sg_u8SysexReceive));
			sg_hMIDIInHdr.dwBufferLength = sizeof(sg_u8SysexReceive);
			sg_hMIDIInHdr.lpData = sg_u8SysexReceive;
			sg_hMIDIInHdr.dwBytesRecorded = 0;
			MIDIResult = midiInAddBuffer(sg_hMIDIInHandle, &sg_hMIDIInHdr, sizeof(sg_hMIDIInHdr));

			if (MIDIResult != MMSYSERR_NOERROR)
			{
				FatalError("midiInAddBuffer returned %.8lx\n", MIDIResult);
				exit(1);
			}
		}
	}
}

/*****************************************************************************
 *
 * Procedure  : OpenMIDICommunication
 *
 * Input      : Nothing
 *
 * Output     : Nothing
 *
 * Description:
 *
 * Opens up the MIDI buses defined in sg_u32InputBus and sg_u32OutputBus.
 *
 *****************************************************************************/

static BOOL OpenMIDICommunication(void)
{
	UINT8 u8String[200];
	MMRESULT MIDIResult;

	// Things are ready to go. Let's open the input and output buses

	MIDIResult = midiInOpen(&sg_hMIDIInHandle, sg_u32InputBus, (DWORD) MIDIReceive, 0, CALLBACK_FUNCTION);

	if (MIDIResult != MMSYSERR_NOERROR)
	{
		sprintf(u8String, "midiInOpen returned %.8lx. Is the MIDI port currently in use?\n", MIDIResult);
		HostPostError(u8String);
		return(FALSE);
	}

	memset(&sg_hMIDIInHdr, 0, sizeof(sg_hMIDIInHdr));
	sg_hMIDIInHdr.dwBufferLength = sizeof(sg_u8SysexReceive);
	sg_hMIDIInHdr.lpData = sg_u8SysexReceive;

	MIDIResult = midiInPrepareHeader(sg_hMIDIInHandle, &sg_hMIDIInHdr, sizeof(sg_hMIDIInHdr));

	if (MIDIResult != MMSYSERR_NOERROR)
	{
		midiInClose(sg_hMIDIInHandle);

		sprintf(u8String, "midiInPrepareHeader returned %.8lx\n", MIDIResult);
		HostPostError(u8String);
		return(FALSE);
	}
	
	MIDIResult = midiInAddBuffer(sg_hMIDIInHandle, &sg_hMIDIInHdr, sizeof(sg_hMIDIInHdr));

	if (MIDIResult != MMSYSERR_NOERROR)
	{
		midiInClose(sg_hMIDIInHandle);
		sprintf(u8String, "midiInAddBuffer returned %.8lx\n", MIDIResult);
		HostPostError(u8String);
		return(FALSE);
	}
	
	midiInStart(sg_hMIDIInHandle);

	MIDIResult = midiOutOpen(&sg_hMIDIOutHandle, sg_u32OutputBus, 0, 0, CALLBACK_NULL);

	if (MIDIResult != MMSYSERR_NOERROR)
	{
		midiInStop(sg_hMIDIInHandle);
		midiInClose(sg_hMIDIInHandle);
		sprintf(u8String, "midiOutOpen returned %.8lx. Is the MIDI port currently in use?\n", MIDIResult);
		HostPostError(u8String);
		return(FALSE);
	}

	return(TRUE);
}

/*****************************************************************************
 *
 * Procedure  : CloseMIDICommunication
 *
 * Input      : Nothing
 *
 * Output     : Nothing
 *
 * Description:
 *
 * Stops all MIDI communication (if any).
 *
 *****************************************************************************/

static void CloseMIDICommunication(void)
{
	// We don't care about return codes

	(void) midiOutClose(sg_hMIDIOutHandle);
	(void) midiInStop(sg_hMIDIInHandle);
	(void) midiInReset(sg_hMIDIInHandle);
	(void) midiInUnprepareHeader(sg_hMIDIInHandle, &sg_hMIDIInHdr, sizeof(sg_hMIDIInHdr));
	(void) midiInClose(sg_hMIDIInHandle);

	sg_hMIDIInHandle = 0;
	sg_hMIDIOutHandle = 0;
}

/*****************************************************************************
 *
 * Procedure  : CheckFileIntegrity
 *
 * Input      : Nothing
 *
 * Output     : Nothing
 *
 * Description:
 *
 * Checks the file integrity for a valid Europa image (passed in as pu8Filename).
 * If the file is good, the UPDATE button is enabled. If it's bad, the UPDATE
 * button is cleared and an error message is displayed.
 *
 *****************************************************************************/

static void CheckFileIntegrity(UINT8 *pu8Filename)
{
	FILE *fp;
	UINT32 u32Size;
	UINT8 *pu8Buffer = NULL;
	EEuropaError eError;
	UINT8 u8Temp[300];

	if (FALSE == sg_bConnected)
	{
		SetUpdateButton(FALSE);
		return;
	}

	if ('\0' == sg_u8EFIFilename[0])
	{
		SetUpdateButton(FALSE);
		return;
	}

	fp = fopen(sg_u8EFIFilename, "rb");

	if (NULL == fp)
	{
		sprintf(u8Temp, "Can't open file %s for reading", sg_u8EFIFilename);
		HostPostError(u8Temp);
		SetUpdateButton(FALSE);
	}
	else
	{
		fseek(fp, 0, SEEK_END);
		u32Size = ftell(fp);
		pu8Buffer = malloc(u32Size);

		if (NULL == pu8Buffer)
		{
			sprintf(u8Temp, "Can't allocate %ld bytes!", u32Size);
			HostPostError(u8Temp);
			SetUpdateButton(FALSE);
			return;
		}
		else
		{
			fseek(fp, 0, SEEK_SET);
			fread(pu8Buffer, 1, u32Size, fp);
		}

		fclose(fp);

		eError = EuropaCheckFileIntegrity(pu8Buffer, u32Size);

		if (eError != EUROPA_NO_ERROR)
		{
			free(pu8Buffer);
			HostPostError((UINT8 *) EuropaGetText(eError+ETXT_EUROPA_ERROR_BASE));
			SetUpdateButton(FALSE);
		}
		else
		{
			SetUpdateButton(TRUE);
		}
	}
}

/*****************************************************************************
 *
 * Procedure  : RegistryMidiPortNumber
 *
 * Input      : TRUE to write data or FALSE to read data
 *
 * Output     : BOOL -- Whether or not the operation was successful.
 *
 * Description:
 *
 * Method to read/write the midi port number from/to the registry.  If the
 * values don't exist in the registry, the default values will be used.
 *
 *****************************************************************************/

BOOL RegistryMidiPortNumber(BOOL bWrite)
{
	DWORD   dwValue = 0;
	DWORD   dwValueSize = sizeof(DWORD);
	LONG    nRetCode = 0;
	HKEY    hKey = NULL;
	DWORD   dwCreated = 0;
	UINT8   u8Temp[300];

	// Open or create the key: HKLM\Software\Synthcom\Europa
	nRetCode = RegCreateKeyEx(HKEY_LOCAL_MACHINE,
							  "SOFTWARE\\SynthcomSystemsInc\\Europa",
							  0, NULL, REG_OPTION_NON_VOLATILE,
							  KEY_ALL_ACCESS, NULL, &hKey, &dwCreated);
	if (ERROR_SUCCESS != nRetCode)
	{
		SetUpdateButton(FALSE);
		return FALSE;
	}
	
	// If bWrite == TRUE, set the bus values to the registry
	// Else if bWrite == FALSE, get the bus values from the registry
	if (bWrite)
	{
		// Write the values to the registry
		nRetCode = RegSetValueEx(hKey, "InputBus", 0, REG_DWORD, (LPBYTE) &sg_u32InputBus, sizeof(sg_u32InputBus));
		if (ERROR_SUCCESS != nRetCode)
		{
			sprintf(u8Temp, "Unable to set the Input Bus info to the registry.");
			HostPostError(u8Temp);
			SetUpdateButton(FALSE);
			return FALSE;
		}
			
		nRetCode = RegSetValueEx(hKey, "OutputBus", 0, REG_DWORD, (LPBYTE) &sg_u32OutputBus, sizeof(sg_u32OutputBus));
		if (ERROR_SUCCESS != nRetCode)
		{
			sprintf(u8Temp, "Unable to set the Output Bus info to the registry.");
			HostPostError(u8Temp);			
			SetUpdateButton(FALSE);
			return FALSE;
		}	
	}
	else
	{	
		// Read the values from the registry
		nRetCode = RegQueryValueEx(hKey, "InputBus", NULL, NULL, (LPBYTE) &sg_u32InputBus, &dwValueSize);
		if (ERROR_SUCCESS != nRetCode)
		{
			SetUpdateButton(FALSE);
			return FALSE;
		}
			
		nRetCode = RegQueryValueEx(hKey, "OutputBus", NULL, NULL, (LPBYTE) &sg_u32OutputBus, &dwValueSize);
		if (ERROR_SUCCESS != nRetCode)
		{
			SetUpdateButton(FALSE);
			return FALSE;
		}

		// Set the current selection

		(void) SendDlgItemMessage(sg_hDialog, IDC_MIDI_IN_BUS, CB_SETCURSEL, sg_u32InputBus, 0);
		(void) SendDlgItemMessage(sg_hDialog, IDC_MIDI_OUT_BUS, CB_SETCURSEL, sg_u32OutputBus, 0);
	}

	// Close the handle to the registry
	RegCloseKey(hKey);

	return TRUE;
}


/*****************************************************************************
 *
 * Procedure  : DlgProc
 *
 * Input      : Dialog box owner, message #, and parameters
 *
 * Output     : Nothing
 *
 * Description:
 *
 * Message handler for main dialog.
 *
 *****************************************************************************/

static long WINAPI DlgProc(HWND hDlg, unsigned u32message, UINT wParam, LONG lParam)
{
	UINT8 u8Temp[300];
	EEuropaError eError;

	switch (u32message)
	{
		case WM_INITDIALOG:
		{
			sg_hDialog = hDlg;
			SetUpdateButton(FALSE);
			SetFindImageButton(FALSE);
			SetConnectButton(FALSE);
			
			if (0 == HostProbeMIDIBuses())
			{
				exit(1);
			}

			// Read the MIDI port numbers from the registry.
			RegistryMidiPortNumber(FALSE);

			return(TRUE);
		};

		case WM_CLOSE:
		{		
			// Write the MIDI port numbers from the registry.
			RegistryMidiPortNumber(TRUE);

			EndDialog(hDlg, 0);
			break;
		};

		case WM_COMMAND:
		{
			RegistryMidiPortNumber(TRUE);

			// If this is true, then we've just changed focus off the filename. Time to
			// check it out!
	
			if ((LOWORD(wParam) == IDC_FILENAME) && (HIWORD(wParam) == EN_KILLFOCUS))
			{
				GetDlgItemText(hDlg, IDC_FILENAME, (LPTSTR) sg_u8EFIFilename, sizeof(sg_u8EFIFilename) - 1);
				CheckFileIntegrity(sg_u8EFIFilename);
			}

			// If we've changed one of the MIDI buses, kill the find/update image buttons until
			// a connect has been established!

			if ( ((LOWORD(wParam) == IDC_MIDI_IN_BUS) || (LOWORD(wParam) == IDC_MIDI_OUT_BUS)) &&
				 (CBN_SELCHANGE == HIWORD(wParam)) )
			{
				sg_bConnected = FALSE;
				SetFindImageButton(FALSE);
				SetUpdateButton(FALSE);
			}

			if (ID_EXIT == wParam)
			{
				PostMessage(hDlg, WM_CLOSE, 0, 0);
			}

			if (ID_FIND_IMAGE == wParam)
			{
				if (FALSE == sg_bConnected)
				{
					HostPostError("Establish a connection first!");
					break;
				}
				else
				{
					OpenFileDialog();
					GetDlgItemText(hDlg, IDC_FILENAME, (LPTSTR) sg_u8EFIFilename, sizeof(sg_u8EFIFilename) - 1);
					CheckFileIntegrity(sg_u8EFIFilename);
				}
			}

			sg_u32InputBus = SendDlgItemMessage(sg_hDialog, IDC_MIDI_IN_BUS, CB_GETCURSEL, 0, 0);
			sg_u32OutputBus = SendDlgItemMessage(sg_hDialog, IDC_MIDI_OUT_BUS, CB_GETCURSEL, 0, 0);

			if ((0xffffffff == sg_u32InputBus) || (sg_u32InputBus < 1) ||
				(0xffffffff == sg_u32OutputBus) || (sg_u32OutputBus < 1))
			{
				SetConnectButton(FALSE);
			}
			else
			{
				SetConnectButton(TRUE);
			}

			if (ID_CONNECT == wParam)
			{
				CloseMIDICommunication();

				if ((0xffffffff == sg_u32OutputBus) || (sg_u32OutputBus < 1))
				{
					HostPostError("MIDI Transmit bus not selected");
					break;
				}
				else
				if ((0xffffffff == sg_u32InputBus) || (sg_u32InputBus < 1))
				{
					HostPostError("MIDI Receive bus not selected");
					break;
				}

				sg_u32InputBus--;
				sg_u32OutputBus--;

				// Let's open up communication, shall we?

				if (OpenMIDICommunication() == FALSE)
				{
					break;
				}

				sg_bConnected = FALSE;

				GetDlgItemText(hDlg, IDC_FILENAME, (LPTSTR) sg_u8EFIFilename, sizeof(sg_u8EFIFilename) - 1);
				eError = EuropaEstablishConnection(&sStatus);

				if (eError != EUROPA_NO_ERROR)
				{
					CloseMIDICommunication();
					SetFindImageButton(FALSE);
					SetUpdateButton(FALSE);
					HostPostError((UINT8 *) EuropaGetText(eError+ETXT_EUROPA_ERROR_BASE));
				}
				else
				{
					SetDlgItemText(sg_hDialog, IDC_SERIAL_NUMBER, sStatus.u8SerialNumber);
					SetDlgItemText(sg_hDialog, IDC_VERSION_NUMBER, sStatus.u8EuropaVersionText);
					sg_bConnected = TRUE;

					MessageBox(sg_hDialog, "Connection established!", "Europa Firmware Update Utility", MB_OK);
					SetFindImageButton(TRUE);
					SetUpdateButton(FALSE);

					CheckFileIntegrity(sg_u8EFIFilename);
				}
			}

			if (ID_UPDATE == wParam)
			{
				FILE *fp;
				UINT32 u32Size;
				UINT8 *pu8Buffer = NULL;
				EEuropaError eError;

				fp = fopen(sg_u8EFIFilename, "rb");

				if (NULL == fp)
				{
					sprintf(u8Temp, "Can't open file %s for reading", sg_u8EFIFilename);
					HostPostError(u8Temp);
				}
				else
				{
					fseek(fp, 0, SEEK_END);
					u32Size = ftell(fp);
					pu8Buffer = malloc(u32Size);

					if (NULL == pu8Buffer)
					{
						sprintf(u8Temp, "Can't allocate %ld bytes!", u32Size);
					}
					else
					{
						fseek(fp, 0, SEEK_SET);
						fread(pu8Buffer, 1, u32Size, fp);
					}

					fclose(fp);

					eError = EuropaCheckFileIntegrity(pu8Buffer, u32Size);

					if (eError != EUROPA_NO_ERROR)
					{
						free(pu8Buffer);
						HostPostError((UINT8 *) EuropaGetText(eError+ETXT_EUROPA_ERROR_BASE));
					}
					else
					{
						eError = EuropaApplyImage(pu8Buffer, u32Size, &sStatus);

						if (EUROPA_NO_ERROR == eError)
						{
							SetDlgItemText(sg_hDialog, IDC_SERIAL_NUMBER, sStatus.u8SerialNumber);
							SetDlgItemText(sg_hDialog, IDC_VERSION_NUMBER, sStatus.u8EuropaVersionText);
							MessageBox(sg_hDialog, "Europa was successfully updated.", "Europa Firmware Update Utility", MB_OK);
						}
						else
						{
							HostPostError((UINT8 *) EuropaGetText(eError+ETXT_EUROPA_ERROR_BASE));
						}
					}
				}

				if (pu8Buffer)
				{				
					free(pu8Buffer);
				}
			}
		};

		default:
		{
		};
	}

	// Default message handler

	return(FALSE);
}

/*****************************************************************************
 *
 * Procedure  : DlgProgressProc
 *
 * Input      : Dialog to update, message, and messages
 *
 * Output     : Nothing
 *
 * Description:
 *
 * Handles all progress dialog messages.
 *
 *****************************************************************************/

static long WINAPI DlgProgressProc(HWND hDlg, unsigned u32message, UINT wParam, LONG lParam)
{
	switch (u32message)
	{
		case WM_INITDIALOG:
		{
			sg_hProgressDialog = hDlg;
			SendDlgItemMessage(sg_hProgressDialog, IDC_PROGRESS1, PBM_SETRANGE, 0, MAKELPARAM(0, 100));
			return(TRUE);
		};

		case WM_COMMAND:
		{
			if (IDC_CANCEL == wParam)
			{
				EuropaCancel();
			}

			break;
		};

		case WM_CLOSE:
		{		
			EndDialog(sg_hProgressDialog, 0);
			break;
		};
	}

	return(FALSE);
}

static HANDLE sg_hProgressThread;

/*****************************************************************************
 *
 * Procedure  : ProgressThread
 *
 * Input      : Thread parameter (not used)
 *
 * Output     : Nothing
 *
 * Description:
 *
 * Executed when the dialog box thread is created, and creates a progress dialog
 * box.
 *
 *****************************************************************************/

static DWORD WINAPI ProgressThread(LPVOID vpParameter)
{
	DialogBox(sg_hInstance, "PROGRESS", 0, DlgProgressProc);
	return(0);
}

/*****************************************************************************
 *
 * Procedure  : HostBeginProgress
 *
 * Input      : Progress dialog box title
 *
 * Output     : Nothing
 *
 * Description:
 *
 * Initializes and shows the progress dialog box by spawning off the progress
 * dialog box thread.
 *
 *****************************************************************************/

void HostBeginProgress(const UINT8 *pu8ProgressTitle)
{
	UINT32 u32ThreadID;

	sg_hProgressThread = CreateThread(NULL, 0, ProgressThread, NULL, 0, &u32ThreadID);
}	

/*****************************************************************************
 *
 * Procedure  : HostEndProgress
 *
 * Input      : Nothing
 *
 * Output     : Nothing
 *
 * Description:
 *
 * Shuts down the progress dialog box thread (and dialog box, too)
 *
 *****************************************************************************/

void HostEndProgress(void)
{
	TerminateThread(sg_hProgressThread, 0);	
}

/*****************************************************************************
 *
 * Procedure  : HostShowProgress
 *
 * Input      : Progress string (what's it doing?) and % complete
 *
 * Output     : Nothing
 *
 * Description:
 *
 * Updates the progress dialog box's name and percentage complete
 *
 *****************************************************************************/

void HostShowProgress(UINT8 *pu8ProgressString,
					  UINT32 u32PercentComplete)
{
	SetDlgItemText(sg_hProgressDialog, IDC_ACTIVITY, pu8ProgressString);
	SendDlgItemMessage(sg_hProgressDialog, IDC_PROGRESS1, PBM_SETPOS, u32PercentComplete, 0);
}

/*****************************************************************************
 *
 * Procedure  : HostSleep
 *
 * Input      : # Of 10ths of seconds to delay
 *
 * Output     : Nothing
 *
 * Description:
 *
 * Puts the calling thread to sleep for a count of 10ths of a second.
 *
 *****************************************************************************/

void HostSleep(UINT8 u8TenthsOfSeconds)
{
	Sleep((UINT32) u8TenthsOfSeconds * 100);
}

/*****************************************************************************
 *
 * Procedure  : HostSendMIDIBlock
 *
 * Input      : Pointer to MIDI data to send and length to send
 *
 * Output     : Nothing
 *
 * Description:
 *
 * Sends a block of data bytes out the currently active MIDI out bus.
 *
 *****************************************************************************/

void HostSendMIDIBlock(UINT8 *pu8MIDIData,
					   UINT32 u32Length)
{
	MIDIHDR mHdr;
	MMRESULT mmresult;
	UINT8 *pu8Buffer;

	pu8Buffer = pu8MIDIData;

	memset(&mHdr, 0, sizeof(mHdr));

	mHdr.dwBufferLength = u32Length;
	mHdr.lpData = pu8MIDIData;
	mHdr.dwBytesRecorded = u32Length;

	mmresult = midiOutPrepareHeader(sg_hMIDIOutHandle, &mHdr, sizeof(mHdr));

	if (mmresult != MMSYSERR_NOERROR)
	{
		FatalError("midiOutPrepareHeader call failed - code %.8lx\n", mmresult);
		exit(1);
	}

	mmresult = midiOutLongMsg(sg_hMIDIOutHandle, &mHdr, sizeof(mHdr));

	if (mmresult != MMSYSERR_NOERROR)
	{
		FatalError("midiOutLongMsg call failed - code %.8lx\n", mmresult);
		exit(1);
	}

	mmresult = midiOutUnprepareHeader(sg_hMIDIOutHandle, &mHdr, sizeof(mHdr));

	while (mmresult == MIDIERR_STILLPLAYING)
	{
		mmresult = midiOutUnprepareHeader(sg_hMIDIOutHandle, &mHdr, sizeof(mHdr));

		Sleep(100);
	}

	if (mmresult != MMSYSERR_NOERROR)
	{
		FatalError("midiOutUnprepareHeader call failed - code %.8lx\n", mmresult);
		exit(1);
	}
}

/*****************************************************************************
 *
 * Procedure  : WinMain
 *
 * Input      : Regular Windows entry point
 *
 * Output     : Nothing
 *
 * Description:
 *
 * Main entry point for ALL activity for this application!
 *
 *****************************************************************************/

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	sg_hInstance = hInstance;
	DialogBox(hInstance, "DIALOG", HWND_DESKTOP, DlgProc);
	CloseMIDICommunication();
	return(0);
}

